@@ -20,11 +20,13 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
+	"net/http"
 	"os"
 	"path/filepath"
 	"sort"
 	"testing"
 
+	"github.com/pkg/errors"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"gopkg.in/yaml.v3"
@@ -347,3 +349,149 @@ func RunCommand(t *testing.T, name string, cwd string, exec string, args ...stri
 		t.FailNow()
 	}
 }
+
+type SchemaVersion = string
+
+const (
+	AwsSchema         SchemaVersion = "4.21.1"
+	AzureNativeSchema SchemaVersion = "1.29.0"
+	AzureSchema       SchemaVersion = "4.18.0"
+	KubernetesSchema  SchemaVersion = "3.7.2"
+	RandomSchema      SchemaVersion = "4.2.0"
+)
+
+var schemaVersions = map[string]struct {
+	version SchemaVersion
+	url     string
+}{
+	"aws.json": {
+		version: AwsSchema,
+		url:     "https://raw.githubusercontent.com/pulumi/pulumi-aws/v%s/provider/cmd/pulumi-resource-aws/schema.json", //nolint:lll
+	},
+	"azure.json": {
+		version: AzureSchema,
+		url:     "https://raw.githubusercontent.com/pulumi/pulumi-azure/v%s/provider/cmd/pulumi-resource-azure/schema.json",
+	},
+	"azure-native.json": {
+		version: AzureNativeSchema,
+		url:     "https://raw.githubusercontent.com/pulumi/pulumi-azure-native/v%s/provider/cmd/pulumi-resource-azure-native/schema.json", //nolint:lll
+	},
+	"kubernetes.json": {
+		version: KubernetesSchema,
+		url:     "https://raw.githubusercontent.com/pulumi/pulumi-kubernetes/v%s/provider/cmd/pulumi-resource-kubernetes/schema.json", //nolint:lll
+	},
+	"random.json": {
+		version: RandomSchema,
+		url:     "https://raw.githubusercontent.com/pulumi/pulumi-random/v%s/provider/cmd/pulumi-resource-random/schema.json",
+	},
+}
+
+// ensureValidSchemaVersions ensures that we have downloaded valid schema for
+// the tests. If it does not find such schema, or the schema found have the
+// wrong version, the function downloads correct schema.
+func ensureValidSchemaVersions(t *testing.T) {
+	c := make(chan error)
+	updates := 0
+	for k, v := range schemaVersions {
+		path := filepath.Join("..", "internal", "test", "testdata", k)
+		current, err := currentVersion(path)
+		var isJSONSyntaxError bool
+		if err != nil {
+			_, isJSONSyntaxError = err.(*json.SyntaxError)
+		}
+		if os.IsNotExist(err) || isJSONSyntaxError || (err == nil && current != v.version) {
+			t.Logf("Updating %s from %s to %s", k, current, v.version)
+			updates++
+			go replaceSchema(c, path, v.version, v.url)
+		} else if err != nil {
+			t.Errorf("failed to get schema version: %s", err)
+		}
+	}
+	var err error
+	for i := 0; i < updates; i++ {
+		err = <-c
+		if err != nil {
+			t.Errorf("failed to update schema: %s", err)
+		}
+	}
+	if err != nil {
+		t.FailNow()
+	}
+}
+
+// Reads the current version of the installed package schema
+func currentVersion(path string) (string, error) {
+	str, err := ioutil.ReadFile(path)
+	if err != nil {
+		return "", err
+	}
+	var data interface{}
+	err = json.Unmarshal(str, &data)
+	if err != nil {
+		return "", err
+	}
+	json, ok := data.(map[string]interface{})
+	if !ok {
+		return "", errors.Errorf("%s could not be read", path)
+	}
+	version, ok := json["version"]
+	if !ok {
+		return "", errors.New("Could not find version field")
+	}
+	versionString, ok := version.(string)
+	if !ok {
+		return "", errors.New("version value is not a string")
+	}
+	return versionString, nil
+}
+
+// Replaces the installed package schema with one containing the correct version.
+func replaceSchema(c chan error, path, version, url string) {
+	// This is safe because url is always a reference to a page Pulumi
+	// controls in github.
+	resp, err := http.Get(fmt.Sprintf(url, version)) //nolint:gosec
+	if err != nil {
+		c <- err
+		return
+	}
+	defer resp.Body.Close()
+
+	err = os.Remove(path)
+	if !os.IsNotExist(err) && err != nil {
+		c <- errors.Wrap(err, "failed to replace schema")
+		return
+	}
+	schemaFile, err := os.Create(path)
+	if err != nil {
+		c <- err
+		return
+	}
+	defer schemaFile.Close()
+	var schemaRaw interface{}
+	decoder := json.NewDecoder(resp.Body)
+	err = decoder.Decode(&schemaRaw)
+	if err != nil {
+		c <- err
+		return
+	}
+	schema, ok := schemaRaw.(map[string]interface{})
+	if !ok {
+		c <- errors.New("failed to convert schema to map")
+		return
+	}
+	schema["version"] = version
+	encoded, err := json.MarshalIndent(schema, "", "  ")
+	if err != nil {
+		c <- err
+		return
+	}
+
+	written, err := schemaFile.Write(encoded)
+	if err != nil {
+		c <- err
+	} else if written != len(encoded) {
+		c <- errors.New("failed to write full message")
+	} else {
+		c <- nil
+	}
+}
