@@ -1,6 +1,7 @@
 package test
 
 import (
+	"flag"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -11,6 +12,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/cmdutil"
 )
 
 // Defines an extra check logic that accepts the directory with the
@@ -42,88 +44,138 @@ const (
 
 var sdkTests = []sdkTest{
 	{
-		Directory:   "input-collision",
+		Directory:   "naming-collisions",
 		Description: "Schema with types that could potentially produce collisions (go).",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "dash-named-schema",
 		Description: "Simple schema with a two part name (foo-bar)",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "external-resource-schema",
 		Description:      "External resource schema",
 		SkipCompileCheck: codegen.NewStringSet(nodejs, golang),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "nested-module",
 		Description:      "Nested module",
 		SkipCompileCheck: codegen.NewStringSet(dotnet, nodejs),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "nested-module-thirdparty",
 		Description:      "Third-party nested module",
 		SkipCompileCheck: codegen.NewStringSet(dotnet, nodejs),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "plain-schema-gh6957",
 		Description: "Repro for #6957",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "resource-args-python-case-insensitive",
 		Description: "Resource args with same named resource and type case insensitive",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "resource-args-python",
 		Description: "Resource args with same named resource and type",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-enum-schema",
 		Description: "Simple schema with enum types",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-plain-schema",
 		Description: "Simple schema with plain properties",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-plain-schema-with-root-package",
 		Description: "Simple schema with root package set",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-resource-schema",
 		Description: "Simple schema with local resource properties",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-resource-schema-custom-pypackage-name",
 		Description: "Simple schema with local resource properties and custom Python package name",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "simple-methods-schema",
 		Description:      "Simple schema with methods",
 		SkipCompileCheck: codegen.NewStringSet(nodejs, dotnet, golang),
+		Skip:             codegen.NewStringSet("python/test"),
+	},
+	{
+		Directory:   "simple-methods-schema-single-value-returns",
+		Description: "Simple schema with methods that return single values",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "simple-yaml-schema",
 		Description: "Simple schema encoded using YAML",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "provider-config-schema",
 		Description:      "Simple provider config schema",
 		SkipCompileCheck: codegen.NewStringSet(dotnet),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "replace-on-change",
 		Description:      "Simple use of replaceOnChange in schema",
 		SkipCompileCheck: codegen.NewStringSet(golang),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:        "resource-property-overlap",
-		Description:      "A resource with the same name as it's property",
+		Description:      "A resource with the same name as its property",
 		SkipCompileCheck: codegen.NewStringSet(dotnet, nodejs),
+		Skip:             codegen.NewStringSet("python/test"),
 	},
 	{
 		Directory:   "hyphen-url",
-		Description: "A resource url with a hyphen in it's path",
+		Description: "A resource url with a hyphen in its path",
+		Skip:        codegen.NewStringSet("python/test"),
 	},
+	{
+		Directory:        "output-funcs",
+		Description:      "Tests targeting the $fn_output helper code generation feature",
+		SkipCompileCheck: codegen.NewStringSet(dotnet),
+	},
+	{
+		Directory:   "cyclic-types",
+		Description: "Cyclic object types",
+		Skip:        codegen.NewStringSet("python/test"),
+	},
+	{
+		Directory:   "regress-node-8110",
+		Description: "Test the fix for pulumi/pulumi#8110 nodejs compilation error",
+		Skip:        codegen.NewStringSet("nodejs/test", "go/test", "python/test", "dotnet/test"),
+	},
+}
+
+var genSDKOnly bool
+
+func NoSDKCodegenChecks() bool {
+	return genSDKOnly
+}
+
+func init() {
+	flag.BoolVar(&genSDKOnly, "sdk.no-checks", false, "when set, skips all post-SDK-generation checks")
+	// NOTE: the testing package will call flag.Parse.
 }
 
 type SDKCodegenOptions struct {
@@ -131,7 +183,7 @@ type SDKCodegenOptions struct {
 	Language string
 
 	// Language-aware code generator; such as `GeneratePackage`.
-	// from `codgen/dotnet`.
+	// from `codegen/dotnet`.
 	GenPackage GenPkgSignature
 
 	// Extra checks for all the tests. They keys of this map are
@@ -195,8 +247,19 @@ type SDKCodegenOptions struct {
 func TestSDKCodegen(t *testing.T, opts *SDKCodegenOptions) { // revive:disable-line
 	testDir := filepath.Join("..", "internal", "test", "testdata")
 
-	for _, tt := range sdkTests {
-		t.Run(tt.Description, func(t *testing.T) {
+	// Motivation for flagging: concerns about memory utilizaion
+	// in CI. It can be a nice feature for developing though.
+	parallel := cmdutil.IsTruthy(os.Getenv("PULUMI_PARALLEL_SDK_CODEGEN_TESTS"))
+
+	for _, sdkTest := range sdkTests {
+		tt := sdkTest // avoid capturing loop variable `sdkTest` in the closure
+		t.Run(tt.Directory, func(t *testing.T) {
+			if parallel {
+				t.Parallel()
+			}
+
+			t.Log(tt.Description)
+
 			dirPath := filepath.Join(testDir, filepath.FromSlash(tt.Directory))
 
 			schemaPath := filepath.Join(dirPath, "schema.json")
@@ -216,6 +279,10 @@ func TestSDKCodegen(t *testing.T, opts *SDKCodegenOptions) { // revive:disable-l
 				}
 			}
 
+			if genSDKOnly {
+				return
+			}
+
 			CopyExtraFiles(t, dirPath, opts.Language)
 
 			// Merge language-specific global and
