@@ -34,7 +34,9 @@ import (
 	"github.com/pkg/errors"
 
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
+	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/tstypes"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+
 	"github.com/pulumi/pulumi/sdk/v3/go/common/diag"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/cmdutil"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
@@ -134,6 +136,9 @@ type modContext struct {
 	modToPkg                map[string]string // Module name -> package name
 	compatibility           string            // Toggle compatibility mode for a specified target.
 	disableUnionOutputTypes bool              // Disable unions in output types.
+
+	// Determine whether to lift single-value method return values
+	liftSingleValueMethodReturns bool
 }
 
 func (mod *modContext) String() string {
@@ -264,71 +269,92 @@ func tokenToFunctionName(tok string) string {
 	return camel(tokenToName(tok))
 }
 
-func (mod *modContext) typeString(t schema.Type, input bool, constValue interface{}) string {
+func (mod *modContext) typeAst(t schema.Type, input bool, constValue interface{}) tstypes.TypeAst {
 	switch t := t.(type) {
 	case *schema.OptionalType:
-		return mod.typeString(t.ElementType, input, constValue) + " | undefined"
+		return tstypes.Union(
+			mod.typeAst(t.ElementType, input, constValue),
+			tstypes.Identifier("undefined"),
+		)
 	case *schema.InputType:
 		typ := mod.typeString(codegen.SimplifyInputUnion(t.ElementType), input, constValue)
 		if typ == "any" {
-			return typ
+			return tstypes.Identifier("any")
 		}
-		return fmt.Sprintf("pulumi.Input<%s>", typ)
+		return tstypes.Identifier(fmt.Sprintf("pulumi.Input<%s>", typ))
 	case *schema.EnumType:
-		return mod.objectType(nil, t.Token, input, false, true)
+		return tstypes.Identifier(mod.objectType(nil, t.Token, input, false, true))
 	case *schema.ArrayType:
-		return mod.typeString(t.ElementType, input, constValue) + "[]"
+		return tstypes.Array(mod.typeAst(t.ElementType, input, constValue))
 	case *schema.MapType:
-		return fmt.Sprintf("{[key: string]: %v}", mod.typeString(t.ElementType, input, constValue))
+		return tstypes.StringMap(mod.typeAst(t.ElementType, input, constValue))
 	case *schema.ObjectType:
-		return mod.objectType(t.Package, t.Token, input, t.IsInputShape(), false)
+		return tstypes.Identifier(mod.objectType(t.Package, t.Token, input, t.IsInputShape(), false))
 	case *schema.ResourceType:
-		return mod.resourceType(t)
+		return tstypes.Identifier(mod.resourceType(t))
 	case *schema.TokenType:
-		return tokenToName(t.Token)
+		return tstypes.Identifier(tokenToName(t.Token))
 	case *schema.UnionType:
 		if !input && mod.disableUnionOutputTypes {
 			if t.DefaultType != nil {
-				return mod.typeString(t.DefaultType, input, constValue)
+				return mod.typeAst(t.DefaultType, input, constValue)
 			}
-			return "any"
+			return tstypes.Identifier("any")
 		}
 
-		elements := make([]string, len(t.ElementTypes))
+		elements := make([]tstypes.TypeAst, len(t.ElementTypes))
 		for i, e := range t.ElementTypes {
-			elements[i] = mod.typeString(e, input, constValue)
+			elements[i] = mod.typeAst(e, input, constValue)
 		}
-		return strings.Join(elements, " | ")
+		return tstypes.Union(elements...)
 	default:
 		switch t {
 		case schema.BoolType:
-			return "boolean"
+			return tstypes.Identifier("boolean")
 		case schema.IntType, schema.NumberType:
-			return "number"
+			return tstypes.Identifier("number")
 		case schema.StringType:
 			if constValue != nil {
-				return fmt.Sprintf("%q", constValue.(string))
+				return tstypes.Identifier(fmt.Sprintf("%q", constValue.(string)))
 			}
-			return "string"
+			return tstypes.Identifier("string")
 		case schema.ArchiveType:
-			return "pulumi.asset.Archive"
+			return tstypes.Identifier("pulumi.asset.Archive")
 		case schema.AssetType:
-			return "pulumi.asset.Asset | pulumi.asset.Archive"
+			return tstypes.Union(
+				tstypes.Identifier("pulumi.asset.Asset"),
+				tstypes.Identifier("pulumi.asset.Archive"),
+			)
 		case schema.JSONType:
 			fallthrough
 		case schema.AnyType:
-			return "any"
+			return tstypes.Identifier("any")
 		}
 	}
-
 	panic(fmt.Errorf("unexpected type %T", t))
 }
 
+func (mod *modContext) typeString(t schema.Type, input bool, constValue interface{}) string {
+	return tstypes.TypeLiteral(tstypes.Normalize(mod.typeAst(t, input, constValue)))
+}
+
 func isStringType(t schema.Type) bool {
 	t = codegen.UnwrapType(t)
 
-	for tt, ok := t.(*schema.TokenType); ok; tt, ok = t.(*schema.TokenType) {
-		t = tt.UnderlyingType
+	switch typ := t.(type) {
+	case *schema.TokenType:
+		t = typ.UnderlyingType
+	case *schema.EnumType:
+		t = typ.ElementType
+	case *schema.UnionType:
+		// The following case detects for relaxed string enums. If it's a Union, check if one ElementType is an EnumType.
+		// If yes, t is the ElementType of the EnumType.
+		for _, tt := range typ.ElementTypes {
+			t = codegen.UnwrapType(tt)
+			if typ, ok := t.(*schema.EnumType); ok {
+				t = typ.ElementType
+			}
+		}
 	}
 
 	return t == schema.StringType
@@ -780,6 +806,8 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		methodName := camel(method.Name)
 		fun := method.Function
 
+		shouldLiftReturn := mod.liftSingleValueMethodReturns && fun.Outputs != nil && len(fun.Outputs.Properties) == 1
+
 		// Write the TypeDoc/JSDoc for the data source function.
 		fmt.Fprint(w, "\n")
 		printComment(w, codegen.FilterExamples(fun.Comment, "typescript"), fun.DeprecationMessage, "    ")
@@ -812,6 +840,8 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		var retty string
 		if fun.Outputs == nil {
 			retty = "void"
+		} else if shouldLiftReturn {
+			retty = fmt.Sprintf("pulumi.Output<%s>", mod.typeString(fun.Outputs.Properties[0].Type, false, nil))
 		} else {
 			retty = fmt.Sprintf("pulumi.Output<%s.%sResult>", name, title(method.Name))
 		}
@@ -829,7 +859,11 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		// Now simply call the runtime function with the arguments, returning the results.
 		var ret string
 		if fun.Outputs != nil {
-			ret = "return "
+			if shouldLiftReturn {
+				ret = fmt.Sprintf("const result: pulumi.Output<%s.%sResult> = ", name, title(method.Name))
+			} else {
+				ret = "return "
+			}
 		}
 		fmt.Fprintf(w, "        %spulumi.runtime.call(\"%s\", {\n", ret, fun.Token)
 		if fun.Inputs != nil {
@@ -843,6 +877,9 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 			}
 		}
 		fmt.Fprintf(w, "        }, this);\n")
+		if shouldLiftReturn {
+			fmt.Fprintf(w, "        return result.%s;\n", camel(fun.Outputs.Properties[0].Name))
+		}
 		fmt.Fprintf(w, "    }\n")
 	}
 	for _, method := range r.Methods {
@@ -1162,10 +1199,14 @@ func (mod *modContext) getImportsForResource(member interface{}, externalImports
 	case *schema.Function:
 		needsTypes := false
 		if member.Inputs != nil {
-			needsTypes = mod.getTypeImports(member.Inputs, false, externalImports, imports, seen) || needsTypes
+			for _, p := range member.Inputs.Properties {
+				needsTypes = mod.getTypeImports(p.Type, false, externalImports, imports, seen) || needsTypes
+			}
 		}
 		if member.Outputs != nil {
-			needsTypes = mod.getTypeImports(member.Outputs, false, externalImports, imports, seen) || needsTypes
+			for _, p := range member.Outputs.Properties {
+				needsTypes = mod.getTypeImports(p.Type, false, externalImports, imports, seen) || needsTypes
+			}
 		}
 		return needsTypes
 	case []*schema.Property:
@@ -1840,15 +1881,10 @@ func genNPMPackageMetadata(pkg *schema.Package, info NodePackageInfo) string {
 		devDependencies["typescript"] = "^4.3.5"
 	}
 
-	packageVersion := "${VERSION}"
-	if pkg.Version != nil {
-		packageVersion = pkg.Version.String()
-	}
-
 	// Create info that will get serialized into an NPM package.json.
 	npminfo := npmPackage{
 		Name:        packageName,
-		Version:     packageVersion,
+		Version:     "${VERSION}",
 		Description: info.PackageDescription,
 		Keywords:    pkg.Keywords,
 		Homepage:    pkg.Homepage,
@@ -1939,6 +1975,8 @@ func genTypeScriptProjectFile(info NodePackageInfo, files fs) string {
 			tsFiles = append(tsFiles, f)
 		}
 	}
+
+	tsFiles = append(tsFiles, info.ExtraTypeScriptFiles...)
 	sort.Strings(tsFiles)
 
 	for i, file := range tsFiles {
@@ -1973,12 +2011,13 @@ func generateModuleContextMap(tool string, pkg *schema.Package, extraFiles map[s
 		mod, ok := modules[modName]
 		if !ok {
 			mod = &modContext{
-				pkg:                     pkg,
-				mod:                     modName,
-				tool:                    tool,
-				compatibility:           info.Compatibility,
-				modToPkg:                info.ModuleToPackage,
-				disableUnionOutputTypes: info.DisableUnionOutputTypes,
+				pkg:                          pkg,
+				mod:                          modName,
+				tool:                         tool,
+				compatibility:                info.Compatibility,
+				modToPkg:                     info.ModuleToPackage,
+				disableUnionOutputTypes:      info.DisableUnionOutputTypes,
+				liftSingleValueMethodReturns: info.LiftSingleValueMethodReturns,
 			}
 
 			if modName != "" {
