@@ -137,6 +137,9 @@ type UpdateOptions struct {
 	// true if the engine should disable resource reference support.
 	DisableResourceReferences bool
 
+	// true if the engine should disable output value support.
+	DisableOutputValues bool
+
 	// true if we should report events for steps that involve default providers.
 	reportDefaultProviderSteps bool
 
