@@ -66,6 +66,7 @@ type provider struct {
 	cfgdone                chan bool                        // closed when configuration has completed.
 	acceptSecrets          bool                             // true if this plugin accepts strongly-typed secrets.
 	acceptResources        bool                             // true if this plugin accepts strongly-typed resource refs.
+	acceptOutputs          bool                             // true if this plugin accepts output values.
 	supportsPreview        bool                             // true if this plugin supports previews for Create and Update.
 	disableProviderPreview bool                             // true if previews for Create and Update are disabled.
 	legacyPreview          bool                             // enables legacy behavior for unconfigured provider previews.
@@ -514,6 +515,7 @@ func (p *provider) Configure(inputs resource.PropertyMap) error {
 		p.acceptSecrets = resp.GetAcceptSecrets()
 		p.acceptResources = resp.GetAcceptResources()
 		p.supportsPreview = resp.GetSupportsPreview()
+		p.acceptOutputs = resp.GetAcceptOutputs()
 
 		p.cfgknown, p.cfgerr = true, err
 		close(p.cfgdone)
@@ -1103,6 +1105,9 @@ func (p *provider) Construct(info ConstructInfo, typ tokens.Type, name tokens.QN
 		KeepUnknowns:  true,
 		KeepSecrets:   p.acceptSecrets,
 		KeepResources: p.acceptResources,
+		// To initially scope the use of this new feature, we only keep output values for
+		// Construct and Call (when the client accepts them).
+		KeepOutputValues: p.acceptOutputs,
 	})
 	if err != nil {
 		return ConstructResult{}, err
@@ -1354,6 +1359,9 @@ func (p *provider) Call(tok tokens.ModuleMember, args resource.PropertyMap, info
 		KeepUnknowns:  true,
 		KeepSecrets:   true,
 		KeepResources: true,
+		// To initially scope the use of this new feature, we only keep output values for
+		// Construct and Call (when the client accepts them).
+		KeepOutputValues: p.acceptOutputs,
 	})
 	if err != nil {
 		return CallResult{}, err
