@@ -1,4 +1,4 @@
-// Copyright 2016-2020, Pulumi Corporation.  All rights reserved.
+// Copyright 2016-2021, Pulumi Corporation.  All rights reserved.
 // +build python all
 
 package ints
@@ -13,7 +13,6 @@ import (
 	"runtime"
 	"strings"
 	"testing"
-	"time"
 
 	"github.com/stretchr/testify/assert"
 
@@ -1090,30 +1089,6 @@ func TestComponentProviderSchemaPython(t *testing.T) {
 	testComponentProviderSchema(t, path, pulumiRuntimeVirtualEnv(t, filepath.Join("..", "..")))
 }
 
-// Regresses an issue with Pulumi hanging when buggy dynamic providers
-// emit outputs that do not match the advertised type.
-func TestBrokenDynamicProvider(t *testing.T) {
-
-	// NOTE: this had some trouble on Windows CI runner with 120
-	// sec max, but passed on a Windows VM locally. IF this
-	// continues to blow the deadline, or be flaky, we should skip
-	// on Windows.
-
-	go func() {
-		<-time.After(600 * time.Second)
-		panic("TestBrokenDynamicProvider: test timed out after 600 seconds, suspect pulumi hanging")
-	}()
-
-	integration.ProgramTest(t, &integration.ProgramTestOptions{
-		Dir: filepath.Join("dynamic", "python-broken"),
-		Dependencies: []string{
-			filepath.Join("..", "..", "sdk", "python", "env", "src"),
-		},
-		Quick:         true,
-		ExpectFailure: true,
-	})
-}
-
 // Test that the about command works as expected. Because about parses the
 // results of each runtime independently, we have an integration test in each
 // language.
